
###### R Code for 'One Covariate at a time Multiple Testing' (OCMT) Function used to generate 2020 election forecasts
###### Make sure to load necessary R libraries, 
###### Make sure to set the working directory to relevant folder
###### Contact: Rashad Ahmed, rashadah@usc.edu

###OCMT function
###se_type takes a value 
### active set requires covariate names


OCMTa<-function(DAT,se_type,y,Z,X,p_val,delta,deltastar){
  ### DAT = panel data set with time/individual dimensions
  ### se_type = value {1,2,3,4,5} to specify which standard errors to apply {least squares, robust, state clustered,year , year-state clustered}
  ### y = dependent variable
  ### Z = regressor (include constant and mandatory factors etc.)  
  ### X = active set  
  ### p_val = p-value cutoff for selection
  ### delta = First stage multiple testing coefficient
  ### deltastar = Additional stage multiple testing coefficient  
  
  TEE=nrow(X)
  N=ncol(X)
  standard_errorz<-c("OCMT Least Squares","OCMT Robust","OCMT State Clustered","OCMT Year Clustered","OCMT State-Year Clustered")
  
  ###stage 1
  ###one by one
  ts=rep(0,N)
  ind=rep(F,N)
  t_threshold=qnorm(1-p_val/2/(N^(delta)),0,1)
  
  for(i in 1:N){
    Xr=as.matrix(cbind(X[,i],Z)) #model matrix
    b=solve(t(Xr) %*% Xr) %*% t(Xr) %*% as.numeric(y) 
    e=y-Xr%*%b
    kr=dim(Xr)[2]   
    ve=(t(e)%*%e)/(TEE-kr)
    App=solve(t(Xr)%*%Xr) 
    
    if(se_type==1){
      
      seb=(App[1,1]*ve)^0.5
    }else if (se_type==2){
      
      u <- matrix(as.numeric(e))
      ustar<-u/(1-diag(Xr%*%App%*%t(Xr))) #Whie & McKannon 1985 Adjustment
      # meat part Sigma is a diagonal with u^2 as elements
      meat1 <- t(Xr) %*% diag(diag(crossprod(t(ustar)))) %*% Xr
      # degrees of freedom adjust
      dfc <- TEE/(TEE-kr)     
      # like before
      seb <- sqrt(dfc*diag(solve(crossprod(Xr)) %*% meat1 %*% solve(crossprod(Xr))))[1]
    } else if (se_type==5){
      
      cluster <- paste(DAT$state,"_",DAT$Year,sep="")
      clusa <- data.frame(Xr,cluster,e)
      colnames(clusa)[(dim(clusa)[2]-1):dim(clusa)[2]] <- c("cluster","resid")
      ma <- dim(table(clusa[,'cluster']))
      dfca <- (ma/(ma-1))*((TEE-1)/(TEE-kr))
      uclusta <- matrix(NA, nrow = ma, ncol = kr)
      gsa <- names(table(clusa[,"cluster"]))
      for(j in 1:ma){
        uclusta[j,] <- t(matrix(clusa[clusa[,"cluster"]==gsa[j],kr+2])) %*% as.matrix(clusa[clusa[,"cluster"]==gsa[j],1:kr] )
      }
      seb<-sqrt(diag(solve(crossprod(Xr)) %*% (t(uclusta) %*% uclusta) %*% solve(crossprod(Xr)))*dfca)[1]
    }else if (se_type==3){
      
      cluster <- paste(DAT$state,sep="")
      clusa <- data.frame(Xr,cluster,e)
      colnames(clusa)[(dim(clusa)[2]-1):dim(clusa)[2]] <- c("cluster","resid")
      ma <- dim(table(clusa[,'cluster']))
      dfca <- (ma/(ma-1))*((TEE-1)/(TEE-kr))
      uclusta <- matrix(NA, nrow = ma, ncol = kr)
      gsa <- names(table(clusa[,"cluster"]))
      for(j in 1:ma){
        uclusta[j,] <- t(matrix(clusa[clusa[,"cluster"]==gsa[j],kr+2])) %*% as.matrix(clusa[clusa[,"cluster"]==gsa[j],1:kr] )
      }
      seb<-sqrt(diag(solve(crossprod(Xr)) %*% (t(uclusta) %*% uclusta) %*% solve(crossprod(Xr)))*dfca)[1]
    } else if (se_type==4){
      
      cluster <- paste(DAT$Year,sep="")
      clusa <- data.frame(Xr,cluster,e)
      colnames(clusa)[(dim(clusa)[2]-1):dim(clusa)[2]] <- c("cluster","resid")
      ma <- dim(table(clusa[,'cluster']))
      dfca <- (ma/(ma-1))*((TEE-1)/(TEE-kr))
      uclusta <- matrix(NA, nrow = ma, ncol = kr)
      gsa <- names(table(clusa[,"cluster"]))
      for(j in 1:ma){
        uclusta[j,] <- t(matrix(clusa[clusa[,"cluster"]==gsa[j],kr+2])) %*% as.matrix(clusa[clusa[,"cluster"]==gsa[j],1:kr] )
      }
      seb<-sqrt(diag(solve(crossprod(Xr)) %*% (t(uclusta) %*% uclusta) %*% solve(crossprod(Xr)))*dfca)[1]
    }
    ts[i]=abs(b[1])/seb
    ind[i]=ts[i]>t_threshold
  }
  
  noadst=0
  
  if (sum(ind)==0){
    do_additional<-F} else{
      do_additional<-T}
  
  ind_first_stage=ind
  
  ### now higher stages (if any)
  t_threshold=qnorm(1-p_val/2/(N^(deltastar)),0,1);
  
  while (do_additional==T){
    ind0=ind
    indb=rep(F,N)  
    
    for( i in 1:N){
      if (ind0[i]==0){
        Xr=as.matrix(cbind(X[,i], X[,ind0], Z)) 
        b=solve(t(Xr) %*% Xr) %*% t(Xr) %*% as.numeric(y) 
        e=y-Xr%*%b
        kr=dim(Xr)[2]   
        ve=(t(e)%*%e)/(TEE-kr)
        iXX=solve(t(Xr)%*%Xr) 
        if(se_type==1){
          seb=(iXX[1,1]*ve)^0.5
        }else if (se_type==2){
          
          u <- matrix(as.numeric(e))
          ustar<-u/(1-diag(Xr%*%iXX%*%t(Xr))) #Whie & McKannon 1985 Adjustment
          # meat part Sigma is a diagonal with u^2 as elements
          meat1 <- t(Xr) %*% diag(diag(crossprod(t(ustar)))) %*% Xr
          # degrees of freedom adjust
          dfc <- TEE/(TEE-kr)     
          # like before
          seb <- sqrt(dfc*diag(solve(crossprod(Xr)) %*% meat1 %*% solve(crossprod(Xr))))[1]
        }else if (se_type==5){
          
          cluster <- paste(DAT$state,"_",DAT$Year,sep="")
          clusa <- data.frame(Xr,cluster,e)
          colnames(clusa)[(dim(clusa)[2]-1):dim(clusa)[2]] <- c("cluster","resid")
          ma <- dim(table(clusa[,'cluster']))
          dfca <- (ma/(ma-1))*((TEE-1)/(TEE-kr))
          uclusta <- matrix(NA, nrow = ma, ncol = kr)
          gsa <- names(table(clusa[,"cluster"]))
          for(j in 1:ma){
            uclusta[j,] <- t(matrix(clusa[clusa[,"cluster"]==gsa[j],kr+2])) %*% as.matrix(clusa[clusa[,"cluster"]==gsa[j],1:kr] )
          }
          seb<-sqrt(diag(solve(crossprod(Xr)) %*% (t(uclusta) %*% uclusta) %*% solve(crossprod(Xr)))*dfca)[1]
        } else if (se_type==3){
          
          cluster <- paste(DAT$state,sep="")
          clusa <- data.frame(Xr,cluster,e)
          colnames(clusa)[(dim(clusa)[2]-1):dim(clusa)[2]] <- c("cluster","resid")
          ma <- dim(table(clusa[,'cluster']))
          dfca <- (ma/(ma-1))*((TEE-1)/(TEE-kr))
          uclusta <- matrix(NA, nrow = ma, ncol = kr)
          gsa <- names(table(clusa[,"cluster"]))
          for(j in 1:ma){
            uclusta[j,] <- t(matrix(clusa[clusa[,"cluster"]==gsa[j],kr+2])) %*% as.matrix(clusa[clusa[,"cluster"]==gsa[j],1:kr] )
          }
          seb<-sqrt(diag(solve(crossprod(Xr)) %*% (t(uclusta) %*% uclusta) %*% solve(crossprod(Xr)))*dfca)[1]
        }else if (se_type==4){
          
          cluster <- paste(DAT$Year,sep="")
          clusa <- data.frame(Xr,cluster,e)
          colnames(clusa)[(dim(clusa)[2]-1):dim(clusa)[2]] <- c("cluster","resid")
          ma <- dim(table(clusa[,'cluster']))
          dfca <- (ma/(ma-1))*((TEE-1)/(TEE-kr))
          uclusta <- matrix(NA, nrow = ma, ncol = kr)
          gsa <- names(table(clusa[,"cluster"]))
          for(j in 1:ma){
            uclusta[j,] <- t(matrix(clusa[clusa[,"cluster"]==gsa[j],kr+2])) %*% as.matrix(clusa[clusa[,"cluster"]==gsa[j],1:kr] )
          }
          seb<-sqrt(diag(solve(crossprod(Xr)) %*% (t(uclusta) %*% uclusta) %*% solve(crossprod(Xr)))*dfca)[1]
        }
        ts[i]=abs(b[1])/seb
        indb[i]=ts[i]>t_threshold
      }
    }
    ind=ind0 | indb;
    do_additional=sum(indb)>0;
    if (do_additional==T){
      noadst=noadst+1;
    }
  }
  
  
  
  ### Now final stage OCMT regression will all selected covariates
  if(length(which(as.numeric(ind)==1))!=0){
  modz<-lm(y~as.matrix(X[,which(as.numeric(ind)==1)]))
  coefz<-as.numeric(coef(modz))
  # get X matrix/predictors
  X2 <- model.matrix(modz)
  # number of obs
  n <- dim(X2)[1]
  # n of predictors
  k <- dim(X2)[2]
  if(se_type==1){
    sez<-summary(modz)$coefficients[,2]
  } else if(se_type==2){
    u <- matrix(resid(modz))
    ustar<-u/(1-diag(X2%*% solve(t(X2)%*%X2) %*%t(X2))) #White & McKannon 1985 Adjustment
    meat1 <- t(X2) %*% diag(diag(crossprod(t(ustar)))) %*% X2
    dfc <- n/(n-k)     
    sez <- as.numeric(sqrt(dfc*diag(solve(crossprod(X2)) %*% meat1 %*% solve(crossprod(X2)))))
  } else if (se_type==3){
    u <- matrix(resid(modz))
    cluster <- paste(DAT$state,sep="")
    clusa <- data.frame(X2,cluster,u)
    colnames(clusa)[(dim(clusa)[2]-1):dim(clusa)[2]] <- c("cluster","resid")
    ma <- dim(table(clusa[,'cluster']))
    dfca <- (ma/(ma-1))*((n-1)/(n-k))
    uclusta <- matrix(NA, nrow = ma, ncol = k)
    gsa <- names(table(clusa[,"cluster"]))
    for(j in 1:ma){
      uclusta[j,] <- t(matrix(clusa[clusa[,"cluster"]==gsa[j],k+2])) %*% as.matrix(clusa[clusa[,"cluster"]==gsa[j],1:k] )
    }
    sez<-as.numeric(sqrt(diag(solve(crossprod(X2)) %*% (t(uclusta) %*% uclusta) %*% solve(crossprod(X2)))*dfca))
  } else if (se_type==5){
    u<-matrix(resid(modz))
    cluster <- paste(DAT$state,"_",DAT$Year,sep="")
    clusa <- data.frame(X2,cluster,u)
    colnames(clusa)[(dim(clusa)[2]-1):dim(clusa)[2]] <- c("cluster","resid")
    ma <- dim(table(clusa[,'cluster']))
    dfca <- (ma/(ma-1))*((n-1)/(n-k))
    uclusta <- matrix(NA, nrow = ma, ncol = k)
    gsa <- names(table(clusa[,"cluster"]))
    for(j in 1:ma){
      uclusta[j,] <- t(matrix(clusa[clusa[,"cluster"]==gsa[j],k+2])) %*% as.matrix(clusa[clusa[,"cluster"]==gsa[j],1:k] )
    }
    sez<-as.numeric(sqrt(diag(solve(crossprod(X2)) %*% (t(uclusta) %*% uclusta) %*% solve(crossprod(X2)))*dfca))
  } else if (se_type==4){
    u<-matrix(resid(modz))
    cluster <- paste(DAT$Year,sep="")
    clusa <- data.frame(X2,cluster,u)
    colnames(clusa)[(dim(clusa)[2]-1):dim(clusa)[2]] <- c("cluster","resid")
    ma <- dim(table(clusa[,'cluster']))
    dfca <- (ma/(ma-1))*((n-1)/(n-k))
    uclusta <- matrix(NA, nrow = ma, ncol = k)
    gsa <- names(table(clusa[,"cluster"]))
    for(j in 1:ma){
      uclusta[j,] <- t(matrix(clusa[clusa[,"cluster"]==gsa[j],k+2])) %*% as.matrix(clusa[clusa[,"cluster"]==gsa[j],1:k] )
    }
    sez<-as.numeric(sqrt(diag(solve(crossprod(X2)) %*% (t(uclusta) %*% uclusta) %*% solve(crossprod(X2)))*dfca))
  }
  
  names(coefz)<-c("Intercept",colnames(X)[which(ind==T)])
  reg_outz<-data.frame(round(coefz,4),round(sez,4),round(coefz/sez,4))
  colnames(reg_outz)<-c("Estimate","SE","tstat")
  }else {
    modz<-lm(y~1)
    coefz<-as.numeric(coef(modz))
    sez<-NA
    reg_outz<-data.frame(round(coefz,4),round(sez,4),round(coefz/sez,4))
  }
    
  
  ret_list<-list(standard_errorz[se_type],as.numeric(ind),sum(as.numeric(ind)),as.numeric(noadst),as.numeric(ind-ind_first_stage),sum(as.numeric(ind-ind_first_stage)),colnames(X)[which(ind==T)],colnames(X)[which(as.numeric(ind-ind_first_stage)==1)],reg_outz,summary(modz)$adj.r.squared,sd(resid(modz)))
  names(ret_list)<-c("Standard Errors","Covariates Selected (All Stages) ","Total Covariates Selected (All Stages)", "Additional OCMT Stages","Covariates Selected (Additional Stages)","Total Covariates Selected (Additional Stages)","All Covariates Selected (Names)","Covariates Selected in Additional Stages (Names)","Final Estimates","R2_Adjusted","Reg_SE")
  return(ret_list)
  
}
